import {
  SubscribeMessage,
  WebSocketGateway,
  OnGatewayInit,
  WebSocketServer,
  OnGatewayConnection,
  OnGatewayDisconnect,
  WsResponse,
} from '@nestjs/websockets';
import { Inject, Logger, UseGuards } from '@nestjs/common';
import { Socket, Server } from 'socket.io';

import { WsJwtGuard } from './auth/guards/ws-auth.guard';
import { AuthService } from './auth/auth.service';
import { PUB_SUB } from './helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';



@WebSocketGateway({
  cors: {
    origin: "*",
    methods: ["GET", "POST"],
    transports: ['websocket', 'polling'],
    credentials: true
  },
  allowEIO3: true
})
//@UseGuards(JwtAuthGuard)
export class AppGateway implements OnGatewayInit, OnGatewayConnection, OnGatewayDisconnect {

  constructor(
 
    
    private readonly authService : AuthService,
    @Inject(PUB_SUB) private pubSub: RedisPubSub
  ) { }

  @WebSocketServer() 
  server: Server;

  private logger: Logger = new Logger('AppGateway');



  afterInit(server: Server) {
    this.logger.log('Init');
  }


  async getActiveInstancesCount() {
    let count = await this.server.of("/").sockets.size;
    //console.log("🚀 ~ file: app.gateway.ts ~ line 53 ~ AppGateway ~ getActiveInstancesCount ~ count", count)

    // send this count with socket to admin
    this.pubSub.publish('ADMIN_HEAD_COUNT',  count );
    //this.clientSocketCollectionService.emitToAdminOnly(count)
    
    return {count : count};
  }

  handleDisconnect(client: Socket) {
    this.logger.log(`Client disconnected: ${client.id}`);
    //this.clientSocketCollectionService.remove(client.id)
    this.getActiveInstancesCount()
  }

  @UseGuards(WsJwtGuard)
  async handleConnection(client: Socket, data: any) {
    let token = client.handshake?.auth?.Authorization;
   
    
    // if we dont want to connect non authorised users, we will check if the token is not null and if the token is valid.
    if(token === null){
      client.disconnect()
      this.logger.log(`Client Disconnect by Auth: ${client.id}`);

      // else we will check if the user is authentic with bearer token
    }else{
      let decoded: any = await this.authService.isValidJwt(token)
      if(decoded){
        this.logger.log(`Client connected: ${client.id}`);
        this.logger.log(`User connected: ${decoded.username}`);
        // this.clientSocketCollectionService.create({
        //   username: decoded.username,
        //   socket: client.id
        // })
        this.getActiveInstancesCount();
      }else{
        client.disconnect()
        this.logger.log(`Client Disconnect by Auth: ${client.id}`);
      }
     
    }

  }




  emitToClient(socket: string, type: string, data: any){
    this.server.to(socket).emit(type, data)
  }

}

