import { Inject, UseGuards } from '@nestjs/common';
import { Resolver, Query, Mutation, Args, Int, Subscription } from '@nestjs/graphql';
import { UserInputError } from 'apollo-server-errors';
import { RedisPubSub } from 'graphql-redis-subscriptions';

import { AppGateway } from './app.gateway';
import { AuthService } from './auth/auth.service';
import { GqlAuthGuard } from './auth/guards/gql-auth.guard';
import { RolesGuard } from './auth/guards/roles.guard';
import { PUB_SUB } from './helper/GlobalConstants';
import { SocketCount, SocketCountSub } from './types/Count.type';
import { LoginSuccessType } from './types/loginSuccess.type';
import { NotificationType } from './types/notification.type';
import { Roles } from './user-collection/decorators/roles.decorator';
import { LoginDto } from './user-collection/dto/login-dto';
import { Role } from './user-collection/roles/role.enum';
import { AppService } from './app.service';
import { DashboardGraphMkCountType } from './types/dashboard-graph-mk-count';

@Resolver()
export class AppResolver {
  constructor(
    private readonly authService: AuthService,
    private readonly appGateway: AppGateway,
    private readonly appService: AppService,
    @Inject(PUB_SUB) private pubSub: RedisPubSub
  ) { }

  @Mutation((returns) => LoginSuccessType)
  async login(
    @Args('username') username: string,
    @Args('password') password: string,
  ) {

    let cr = await this.authService.login({
      username,
      password,
    });



    if (cr.status) {
      return cr;
    } else {
      throw new UserInputError('Username or Password not matched', {
        username: username,
      });
    }
  }


  @Query((returns) => SocketCount)
  socketCount() {
    return this.appGateway.getActiveInstancesCount();

  }




  @Subscription(() => SocketCountSub, {
    resolve: (payload) => {
      //console.log("🚀 ~ file: app.resolver.ts ~ line 50 ~ AppResolver ~ @Subscription ~ payload", payload)
      return { count: payload }
    }
  })
  socketCountSub() {
    //console.log('socketCountSub')
    //let count = await this.appGateway.getActiveInstancesCount();
    return this.pubSub.asyncIterator('ADMIN_HEAD_COUNT');
    //return {count: count}
  }


  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [DashboardGraphMkCountType])
  getCountOfMkRelatedDataForGraph(@Args('year') year?: string) {
    return this.appService.getCountOfMkRelatedDataForGraph(year);
  }
}
