import { Inject, Injectable, forwardRef } from '@nestjs/common';
import { CreateAssessmentAnswerTableInput } from './dto/create-assessment-answer-table.input';
import { UpdateAssessmentAnswerTableInput } from './dto/update-assessment-answer-table.input';
import { InjectRepository } from '@nestjs/typeorm';
import { AssessmentAnswerTable } from './entities/assessment-answer-table.entity';
import { Repository } from 'typeorm';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { AssessmentMasterTableService } from 'src/assessment-master-table/assessment-master-table.service';
import { UserCollectionService } from 'src/user-collection/user-collection.service';
import { AssessmentQuestionsTableService } from 'src/assessment-questions-table/assessment-questions-table.service';
import moment from 'moment';
import { UserTable } from 'src/user-collection/entities/user-collection.entity';
import { AssessmentMasterTable } from 'src/assessment-master-table/entities/assessment-master-table.entity';
import { CourseProgressionTableService } from 'src/course-progression-table/course-progression-table.service';

export const NEW_ANSWER_ADD = "NEW_ANSWER_ADD"

@Injectable()
export class AssessmentAnswerTableService {
  constructor(
    @InjectRepository(AssessmentAnswerTable)
    private readonly assessmentAnswersTableRepository: Repository<AssessmentAnswerTable>,
    @Inject(PUB_SUB) private pubSub: RedisPubSub,
    private readonly assessmentMasterTableService: AssessmentMasterTableService,
    private readonly userCollectionService: UserCollectionService,
    private readonly assessmentQuestionsTableService: AssessmentQuestionsTableService,

    @Inject(forwardRef(() => CourseProgressionTableService))
    private readonly courseProgressionTableService: CourseProgressionTableService,

  ) { }

  async create(createAssessmentAnswerTableInput: CreateAssessmentAnswerTableInput, user: any) {
    let checkIfExistsCount = await this.checkIfExists(createAssessmentAnswerTableInput, user)

    let attempt = 1;

    if (checkIfExistsCount > 0) {
      // we will insert but as attempt+1
      attempt = checkIfExistsCount + 1;
    }

    
    let dd = new AssessmentAnswerTable();

    let assessmentMaster = await this.assessmentMasterTableService.findOne(createAssessmentAnswerTableInput.assessmentId)
    dd.AssessmentMasterTable = assessmentMaster;
    dd.answers = JSON.parse(createAssessmentAnswerTableInput.answers);
    let today = moment().format("DD-MM-YYYY");
    dd.assessmentDate = today;
    // calculate score
    let questions = await this.assessmentQuestionsTableService.findOneByAssessmentMasterId(createAssessmentAnswerTableInput.assessmentId)
    // now we will calculate the score
    let score = 0;

    for (const el of JSON.parse(createAssessmentAnswerTableInput.answers)) {
      let selectedQuestion = questions.find(el1 => el1.assessmentQuestionId === el.assessmentQuestionId)

      if (selectedQuestion) {

        if (el.value === selectedQuestion.correctAnswer) {
          score = score + selectedQuestion.marks
        }
      }
    }

    dd.score = score.toFixed(0);
    dd.attempt = attempt;

    // based on this score identify the interpretation

    if (score >= parseFloat(assessmentMaster.passingScore)) {
      // so the exam is passed
      // store the data to the other table

      await this.courseProgressionTableService.updateCourseProgressionAssessment(assessmentMaster.CourseTable.courseId, user, 1, moment().format("DD-MM-YYYY"), score, 1);

    } else {
      // exam is not passed
      // update the data
      await this.courseProgressionTableService.updateCourseProgressionAssessment(assessmentMaster.CourseTable.courseId, user, 1, moment().format("DD-MM-YYYY"), score, 0);
    }




    let userData = await this.userCollectionService.findOneByUsername(user.username)
    dd.UserTable = userData;

    let result = await this.assessmentAnswersTableRepository.save(dd);

    // send to subscriber
    this.pubSub.publish(NEW_ANSWER_ADD, 1);

    return {
      status: true,
      status_text: "Successfully Created",
      data: result
    }

  }

  convertedArray(array: any[]) {
    return array.map(innerArray => {
      const obj = {};
      innerArray.forEach(([key, value]) => {
        obj[key] = value;
      });
      return obj;
    });
  }

  async recalculateScore(answerId: number) {

    let dd = await this.assessmentAnswersTableRepository.findOne({
      where: {
        answerId
      },
      relations: {
        AssessmentMasterTable: true
      }
    })

    if (dd) {

      // calculate score
      let questions = await this.assessmentQuestionsTableService.findOneByAssessmentMasterId(dd.AssessmentMasterTable.assessmentId)

      // now we will calculate the score
      let score = 0;

      let answers = dd.answers

      for (const el of answers) {
        let selectedQuestion = questions.find(el1 => el1.assessmentQuestionId === el.assessmentQuestionId)

        if (selectedQuestion) {
          let selectedAnswers = selectedQuestion.options;

          let selectedOption = selectedAnswers.find(el3 => el3.optionLabel === el.value);

          if (selectedOption) {
            score = score + parseFloat(selectedOption.optionValue)
          }
        }
      }


      dd.score = score.toFixed(0);

      // based on this score identify the interpretation


      let result = await this.assessmentAnswersTableRepository.save(dd);

      return result;
    } else {
      return false;
    }
  }

  async checkIfExists(createAssessmentAnswerTableInput: CreateAssessmentAnswerTableInput, user: any) {

    let result = await this.assessmentAnswersTableRepository.count({
      where: {
        AssessmentMasterTable: {
          assessmentId: createAssessmentAnswerTableInput.assessmentId
        },
        UserTable: {
          id: user.id
        }
      }
    })

    return result;

  }

  async findAll() {
    return await this.assessmentAnswersTableRepository.find()
  }

  async findOne(answerId: number) {
    return await this.assessmentAnswersTableRepository.findOne({
      where: {
        answerId: answerId
      },
      relations: {
        AssessmentMasterTable: true,
        UserTable: {
          DesignationTable: true
        }
      }
    });
  }

  async findMyLatestCourse(assessmentId: number, userId: number) {
    let result = await this.assessmentAnswersTableRepository.findOne({
      where: {
        AssessmentMasterTable: {
          assessmentId
        },
        UserTable: {
          id: userId
        }
      },
      relations: {
        AssessmentMasterTable: true,
        UserTable: {
          DesignationTable: true
        }
      },
      order: {
        answerId: 'DESC'
      }
    });

    return result.score;
  }

  async findAllByCourseIdAndUserId(courseId: number, userId: number) {
    return await this.assessmentAnswersTableRepository.find({
      where: {
        AssessmentMasterTable: {
          CourseTable: {
            courseId
          }
        },
        UserTable: {
          id: userId
        }
      },
      relations: {
        AssessmentMasterTable: {
          CourseTable: true
        }
      }
    });
  }

  async getListOfAnswersByUser(
    limit: number,
    page: number,
    type: string,
    search: string,
    user: any
  ) {
    let data: any = await this.assessmentAnswersTableRepository
      .createQueryBuilder('r')
      .where('r.userId=:userId', { userId: user.id })
      .andWhere('r.type=:type', { type: type })
      .leftJoinAndMapOne('r.UserTable', UserTable, 'ml1', 'ml1.id=r.userId')
      .leftJoinAndMapOne('r.assessmentMasterTable', AssessmentMasterTable, 'ml3', 'ml3.assessmentId=r.assessmentId')
      .take(limit)
      .skip(page);
    if (search != '') {
      let s = `${search}%`;
    }

    data = await data.getManyAndCount();


    let [result, count] = data;



    return {
      data: result,
      total: count,
    };
  }

  // async update(answerId: number, updateAssessmentAnswerTableInput: UpdateAssessmentAnswerTableInput, user: any) {
  //   return `This action updates a #${id} assessmentAnswerTable`;
  // }

  async remove(answerId: number) {
    let result = await this.assessmentAnswersTableRepository.findOne({
      where: {
        answerId: answerId
      }
    })


    if (result) {
      await this.assessmentAnswersTableRepository.remove(result)

      // send to subscriber
      this.pubSub.publish(NEW_ANSWER_ADD, 1);
      return {
        status: true,
        status_text: "Successfully Removed",
        data: null
      }
    } else {
      return {
        status: false,
        status_text: "Not Found",
        data: null
      }
    }
  }
}
