import { Resolver, Query, Mutation, Args, Int, Subscription } from '@nestjs/graphql';
import { AssessmentMasterTableService } from './assessment-master-table.service';
import { AssessmentMasterTable } from './entities/assessment-master-table.entity';
import { CreateAssessmentMasterTableInput } from './dto/create-assessment-master-table.input';
import { UpdateAssessmentMasterTableInput } from './dto/update-assessment-master-table.input';
import { Inject, UseGuards } from '@nestjs/common';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { AssessmentMasterTableReturnType } from './types/assessment-master-table-return.type';
import { AssessmentMasterTableType } from './types/assessment-master-table.type';
import { CountChangedType } from 'src/types/count-changed.type';
import { CurrentUser } from 'src/user-collection/decorators/user.decorator';

export const NEW_ASSESSMENT_ADD = "NEW_ASSESSMENT_ADD"

@Resolver(() => AssessmentMasterTable)
export class AssessmentMasterTableResolver {
  constructor(private readonly assessmentMasterTableService: AssessmentMasterTableService,
    @Inject(PUB_SUB) private pubSub: RedisPubSub
  ) {}

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => AssessmentMasterTableReturnType)
  createAssessmentMasterTable(@Args('createAssessmentMasterTableInput') createAssessmentMasterTableInput: CreateAssessmentMasterTableInput) {
    return this.assessmentMasterTableService.create(createAssessmentMasterTableInput);
  }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [AssessmentMasterTableType])
  findAllAssessments() {
    return this.assessmentMasterTableService.findAll();
  }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => AssessmentMasterTableType)
  findOneAssessment(@Args('assessmentId', { type: () => Int }) assessmentId: number) {
    return this.assessmentMasterTableService.findOne(assessmentId);
  }


  


  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [AssessmentMasterTableType])
  findOneAssessmentByCourseUnitId(@Args('courseId', { type: () => Int }) courseId: number) {
    return this.assessmentMasterTableService.findOneAssessmentByCourseUnitId(courseId);
  }



  @Roles(Role.SUPERADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [AssessmentMasterTableType])
  findOneAssessmentByCourseUnitIdActive(@Args('courseId', { type: () => Int }) courseId: number, @CurrentUser() user: any) {
    return this.assessmentMasterTableService.findOneAssessmentByCourseUnitIdActive(courseId, user);
  }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => AssessmentMasterTableReturnType)
  updateAssessmentMasterTable(@Args('updateAssessmentMasterTableInput') updateAssessmentMasterTableInput: UpdateAssessmentMasterTableInput) {
    return this.assessmentMasterTableService.update(updateAssessmentMasterTableInput.assessmentId, updateAssessmentMasterTableInput);
  }


  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => AssessmentMasterTableReturnType)
  toggleActiveStatusAssesment(
    @Args('assessmentId')
    assessmentId: number,
  ) {
    return this.assessmentMasterTableService.toggleActiveStatus(assessmentId);
  }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => AssessmentMasterTableReturnType)
  removeAssessmentMasterTable(@Args('assessmentId', { type: () => Int }) assessmentId: number) {
    return this.assessmentMasterTableService.remove(assessmentId);
  }

  @Subscription(() => CountChangedType, {
    resolve: (payload) => {
      return { countChanged: payload }
    }
  })
  assessmentSubscription() {
    return this.pubSub.asyncIterator(NEW_ASSESSMENT_ADD);
  }
}
