import { Inject, Injectable, forwardRef } from '@nestjs/common';
import { CreateAssessmentMasterTableInput } from './dto/create-assessment-master-table.input';
import { UpdateAssessmentMasterTableInput } from './dto/update-assessment-master-table.input';
import { InjectRepository } from '@nestjs/typeorm';
import { AssessmentMasterTable } from './entities/assessment-master-table.entity';
import { Repository } from 'typeorm';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { CourseTableService } from 'src/course-table/course-table.service';
import { CourseUnitTableService } from 'src/course-unit-table/course-unit-table.service';
import { CourseProgressionTableService } from 'src/course-progression-table/course-progression-table.service';
import { AssessmentAnswerTableService } from 'src/assessment-answer-table/assessment-answer-table.service';

export const NEW_ASSESSMENT_ADD = "NEW_ASSESSMENT_ADD"

@Injectable()
export class AssessmentMasterTableService {
  constructor(
    @InjectRepository(AssessmentMasterTable)
    private readonly assessmentMasterTableRepository: Repository<AssessmentMasterTable>,
    @Inject(PUB_SUB) private pubSub: RedisPubSub,



    @Inject(forwardRef(() => CourseTableService))
    private readonly courseTableService: CourseTableService,


    @Inject(forwardRef(() => AssessmentAnswerTableService))
    private readonly assessmentAnswerTableService: AssessmentAnswerTableService,

    @Inject(forwardRef(() => CourseProgressionTableService))
    private readonly courseProgressionTableService: CourseProgressionTableService,

  ) { }

  async create(createAssessmentMasterTableInput: CreateAssessmentMasterTableInput) {
    let checkIfExists = await this.checkIfExists(createAssessmentMasterTableInput)


    if (checkIfExists) {
      // error
      return {
        status: false,
        status_text: "Already Exists",
        data: null
      }
    } else {
      let dd = new AssessmentMasterTable();
      dd.assessmentName = createAssessmentMasterTableInput.assessmentName;
      dd.type = createAssessmentMasterTableInput.type;
      dd.totalScore = createAssessmentMasterTableInput.totalScore;
      dd.passingScore = createAssessmentMasterTableInput.passingScore;
      dd.isActive = createAssessmentMasterTableInput.isActive;
      dd.duration = createAssessmentMasterTableInput.duration;
      dd.week = createAssessmentMasterTableInput.week;

      dd.CourseTable = await this.courseTableService.findOne(createAssessmentMasterTableInput.courseId)

      let result = await this.assessmentMasterTableRepository.save(dd);

      // send to subscriber
      this.pubSub.publish(NEW_ASSESSMENT_ADD, 1);

      return {
        status: true,
        status_text: "Successfully Created",
        data: result
      }
    }
  }

  async checkIfExists(createAssessmentMasterTableInput: CreateAssessmentMasterTableInput) {
    let result = await this.assessmentMasterTableRepository.count({
      where: {
        assessmentName: createAssessmentMasterTableInput.assessmentName,
        type: createAssessmentMasterTableInput.type,
        week: createAssessmentMasterTableInput.week,
        totalScore: createAssessmentMasterTableInput.totalScore,
        passingScore: createAssessmentMasterTableInput.passingScore,
        CourseTable: {
          courseId: createAssessmentMasterTableInput.courseId
        }
      }
    })

    if (result == 0) {
      return false;
    } else {
      return true;
    }

  }

  async findAll() {
    return await this.assessmentMasterTableRepository.find()
  }

  async findOne(assessmentId: number) {
    return await this.assessmentMasterTableRepository.findOne({
      where: {
        assessmentId: assessmentId
      },
      relations: {
        CourseTable: true
      }
    });
  }

  async findOneAssessmentByCourseUnitId(courseId: number) {
    return await this.assessmentMasterTableRepository.find({
      where: {
        CourseTable: {
          courseId
        }
      }
    });
  }

  async findOneAssessmentByCourseUnitIdActive(courseId: number, user: any) {
    let result = await this.assessmentMasterTableRepository.find({
      where: {
        CourseTable: {
          courseId
        },
        isActive: true
      },
      relations: {
        CourseTable: true
      }
    });


    let index=0;
    for(const el of result){
      // allowing the assessment
      // let isAllowed = await this.courseProgressionTableService.checkIfAllowedForAssessment(el.CourseTable.courseId, user);
      let score = await this.assessmentAnswerTableService.findMyLatestCourse(el.assessmentId, user.id);
      result[index]['isAllowed'] = true;
      result[index]['score'] = score;
      index++;
    }

    

    return result;
  }

  async update(assessmentId: number, updateAssessmentMasterTableInput: UpdateAssessmentMasterTableInput) {
    let result = await this.assessmentMasterTableRepository.findOne({
      where: {
        assessmentId: assessmentId
      }
    })


    if (result) {
      // update


      result.assessmentName = updateAssessmentMasterTableInput.assessmentName;
      result.type = updateAssessmentMasterTableInput.type;
      result.week = updateAssessmentMasterTableInput.week;
      result.totalScore = updateAssessmentMasterTableInput.totalScore;
      result.passingScore = updateAssessmentMasterTableInput.passingScore;
      result.isActive = updateAssessmentMasterTableInput.isActive;
      result.duration = updateAssessmentMasterTableInput.duration;
      result.CourseTable = await this.courseTableService.findOne(updateAssessmentMasterTableInput.courseId)

      let r = await this.assessmentMasterTableRepository.save(result);

      // send to subscriber
      this.pubSub.publish(NEW_ASSESSMENT_ADD, 1);
      return {
        status: true,
        status_text: "Successfully Updated",
        data: r
      }


    } else {
      // return false;
      return {
        status: false,
        status_text: "Not found",
        data: null
      }
    }
  }

  async toggleActiveStatus(
    assessmentId: number
  ) {

    let u = await this.findOne(assessmentId);

    if (!!u) {

      let oldStatus = u.isActive;

      u.isActive = !oldStatus;

      let newStatus = !oldStatus ? 'Activated' : 'De-Activated';
      let u1 = await this.assessmentMasterTableRepository.save(u);


      return {
        status: true,
        status_text: "Status Changed",
        data: await this.findOne(assessmentId)
      };

    } else {
      return {
        status: false,
        status_text: "Survey not found",
        data: null
      };
    }
  }


  async remove(assessmentId: number) {
    let result = await this.assessmentMasterTableRepository.findOne({
      where: {
        assessmentId: assessmentId
      }
    })


    if (result) {
      await this.assessmentMasterTableRepository.remove(result)

      // send to subscriber
      this.pubSub.publish(NEW_ASSESSMENT_ADD, 1);
      return {
        status: true,
        status_text: "Successfully Removed",
        data: null
      }
    } else {
      return {
        status: false,
        status_text: "Not Found",
        data: null
      }
    }
  }
}
