import { Inject, Injectable } from '@nestjs/common';
import { CreateAssessmentQuestionOptionInput, CreateAssessmentQuestionsTableInput } from './dto/create-assessment-questions-table.input';
import { UpdateAssessmentQuestionsTableInput } from './dto/update-assessment-questions-table.input';
import { InjectRepository } from '@nestjs/typeorm';
import { AssessmentQuestionsTable } from './entities/assessment-questions-table.entity';
import { Repository } from 'typeorm';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { AssessmentMasterTableService } from 'src/assessment-master-table/assessment-master-table.service';

export const NEW_QUESTION_ADD = "NEW_QUESTION_ADD"

@Injectable()
export class AssessmentQuestionsTableService {
  constructor(
    @InjectRepository(AssessmentQuestionsTable)
    private readonly assessmentQuestionsTableRepository: Repository<AssessmentQuestionsTable>,
    @Inject(PUB_SUB) private pubSub: RedisPubSub,
    private readonly assessmentMasterTableService: AssessmentMasterTableService,
  ) { }

  async create(createAssessmentQuestionsTableInput: CreateAssessmentQuestionsTableInput) {
    let checkIfAlreadyExists = await this.checkIfAlreadyExists(createAssessmentQuestionsTableInput);

    if (checkIfAlreadyExists == 0) {

      let st = new AssessmentQuestionsTable();
      st.assessmentQuestionEn = createAssessmentQuestionsTableInput.assessmentQuestionEn;
      st.assessmentQuestionAs = createAssessmentQuestionsTableInput.assessmentQuestionAs;

      st.assessmentQuestionLabel = createAssessmentQuestionsTableInput.assessmentQuestionLabel;
      st.correctAnswer = createAssessmentQuestionsTableInput.correctAnswer;
      st.marks = createAssessmentQuestionsTableInput.marks;

      let assessmentMaster = await this.assessmentMasterTableService.findOne(createAssessmentQuestionsTableInput.assessmentId)

      st.AssessmentMasterTable = assessmentMaster;


      st.options = createAssessmentQuestionsTableInput.options.map(
        (option: CreateAssessmentQuestionOptionInput) => ({
          optionLabel: option.optionLabel,
          optionText: option.optionText,
        })
      );

      let result = await this.assessmentQuestionsTableRepository.save(st);


      // send to subscriber
      this.pubSub.publish(NEW_QUESTION_ADD, 1);


      return {
        status: true,
        status_text: "Successfully Added",
        data: result
      }

    } else {
      return {
        status: false,
        status_text: "Already Exists"
      }
    }
  }

  async checkIfAlreadyExists(createAssessmentQuestionsTableInput: CreateAssessmentQuestionsTableInput) {

    let result = await this.assessmentQuestionsTableRepository.count({
      where: {
        assessmentQuestionEn: createAssessmentQuestionsTableInput.assessmentQuestionEn,
        assessmentQuestionAs: createAssessmentQuestionsTableInput.assessmentQuestionAs,
        assessmentQuestionLabel: createAssessmentQuestionsTableInput.assessmentQuestionLabel,
        AssessmentMasterTable: {
          assessmentId: createAssessmentQuestionsTableInput.assessmentId
        }
        // options: createSurveyQuestionsTableInput.options,
      }
    })

    return result;

  }

  async findAll() {
    return await this.assessmentQuestionsTableRepository.find({
      relations: {
        AssessmentMasterTable: true
      }
    })
  }

  async findOne(assessmentQuestionId: number) {
    return await this.assessmentQuestionsTableRepository.findOne({
      where: {
        assessmentQuestionId: assessmentQuestionId
      },
      relations: {
        AssessmentMasterTable: true
      }
    });
  }

  async findOneByAssessmentMasterId(assessmentId: number) {
    return await this.assessmentQuestionsTableRepository.find({
      where: {
        AssessmentMasterTable: {
          assessmentId: assessmentId
        }
      },
      relations: {
        AssessmentMasterTable: true
      }
    });
  }

  async findOneAssessmentByType(type: string) {
    let result = await this.assessmentQuestionsTableRepository.find({
      where: {
        AssessmentMasterTable: {
          type: type
        }
      },
      relations: {
        AssessmentMasterTable: true
      }
    });

    let assessmentId = 0;

    if (result.length > 0) {
      assessmentId = result[0].AssessmentMasterTable.assessmentId;
    }

    // get the largest 

    let labelArray = result.map(el => el.assessmentQuestionLabel);


    return {
      data: this.shuffleArray(result),
      length: this.findLargest(labelArray),
      assessmentId: assessmentId
    }
  }




  async findOneAssessmentByIdForApp(assessmentId: number) {
    let result = await this.assessmentQuestionsTableRepository.find({
      where: {
        AssessmentMasterTable: {
          assessmentId: assessmentId
        }
      },
      relations: {
        AssessmentMasterTable: true
      }
    });


    let duration = 0;

    if(result.length > 0){
      duration = result[0].AssessmentMasterTable.duration;
    }
    // get the largest 

    let labelArray = result.map(el => el.assessmentQuestionLabel);


    return {
      data: this.shuffleArray(result),
      length: this.findLargest(labelArray),
      assessmentId: assessmentId,
      duration
    }
  }


  findLargest(arr: number[]) {
    if (arr.length === 0) { // if array is empty
      return null;
    }

    let maxNum = arr[0]; // Assume first element is the largest
    for (let i = 1; i < arr.length; i++) {
      if (arr[i] > maxNum) {
        maxNum = arr[i];
      }
    }
    return maxNum;
  }

  shuffleArray(array: any) {
    for (let i = array.length - 1; i > 0; i--) {
      const j = Math.floor(Math.random() * (i + 1));
      [array[i], array[j]] = [array[j], array[i]];
    }
    return array;
  }

  async update(assessmentQuestionId: number, updateAssessmentQuestionsTableInput: UpdateAssessmentQuestionsTableInput) {
    let result = await this.assessmentQuestionsTableRepository.findOne({
      where: {
        assessmentQuestionId: assessmentQuestionId
      }
    })


    if (result) {
      let st = result;
      st.assessmentQuestionEn = updateAssessmentQuestionsTableInput.assessmentQuestionEn;
      st.assessmentQuestionAs = updateAssessmentQuestionsTableInput.assessmentQuestionAs;

      st.assessmentQuestionLabel = updateAssessmentQuestionsTableInput.assessmentQuestionLabel;
      st.correctAnswer = updateAssessmentQuestionsTableInput.correctAnswer;
      st.marks = updateAssessmentQuestionsTableInput.marks;

      let assessmentMaster = await this.assessmentMasterTableService.findOne(updateAssessmentQuestionsTableInput.assessmentId)

      st.AssessmentMasterTable = assessmentMaster;


      st.options = updateAssessmentQuestionsTableInput.options.map(
        (option: CreateAssessmentQuestionOptionInput) => ({
          optionLabel: option.optionLabel,
          optionText: option.optionText,
        })
      );

      let result1 = await this.assessmentQuestionsTableRepository.save(st);

      // send to subscriber
      this.pubSub.publish(NEW_QUESTION_ADD, 1);
      return {
        status: true,
        status_text: "Successfully Updated",
        data: result1
      }
    } else {
      return {
        status: false,
        status_text: "Not Found",
        data: null
      }
    }
  }

  async remove(assessmentQuestionId: number) {
    let result = await this.assessmentQuestionsTableRepository.findOne({
      where: {
        assessmentQuestionId: assessmentQuestionId
      }
    })


    if (result) {
      await this.assessmentQuestionsTableRepository.remove(result)

      // send to subscriber
      this.pubSub.publish(NEW_QUESTION_ADD, 1);
      return {
        status: true,
        status_text: "Successfully Removed",
        data: null
      }
    } else {
      return {
        status: false,
        status_text: "Not Found",
        data: null
      }
    }
  }
}
