import { Inject, Injectable, forwardRef } from '@nestjs/common';
import { CreateCourseSessionTableInput } from './dto/create-course-session-table.input';
import { UpdateCourseSessionTableInput } from './dto/update-course-session-table.input';
import { CourseSessionTable } from './entities/course-session-table.entity';
import { Repository } from 'typeorm';
import { InjectRepository } from '@nestjs/typeorm';
import { CourseTableService } from 'src/course-table/course-table.service';
import moment from 'moment';

@Injectable()
export class CourseSessionTableService {
  constructor(
    @InjectRepository(CourseSessionTable)
    private readonly courseSessionTableRepository: Repository<CourseSessionTable>,

    @Inject(forwardRef(() => CourseTableService))
    private readonly courseTableService: CourseTableService,
  ) { }


  async create(createCourseSessionTableInput: CreateCourseSessionTableInput) {

    const checkIfExists = await this.checkIfExists(createCourseSessionTableInput);

    if (!checkIfExists) {
      let cst = new CourseSessionTable();
      cst.sessionName = createCourseSessionTableInput.sessionName;
      cst.capacity = createCourseSessionTableInput.capacity;
      cst.sessionStartingDate = moment(createCourseSessionTableInput.sessionStartingDate, "YYYY-MM-DD").format("DD-MM-YYYY");
      cst.sessionEndingDate = moment(createCourseSessionTableInput.sessionEndingDate, "YYYY-MM-DD").format("DD-MM-YYYY");
      cst.CourseTable = await this.courseTableService.findOne(createCourseSessionTableInput.courseId)


      await this.courseSessionTableRepository.save(cst);

      return {
        status: true,
        status_text: "Succesfully Created"
      }
    } else {
      return {
        status: false,
        status_text: "Already Exists"
      }
    }

  }


  async checkIfExists(createCourseSessionTableInput: CreateCourseSessionTableInput) {
    let result = await this.courseSessionTableRepository.findOne({
      where: {
        sessionName: createCourseSessionTableInput.sessionName,
        sessionStartingDate: moment(createCourseSessionTableInput.sessionStartingDate, "YYYY-MM-DD").format("DD-MM-YYYY"),
        sessionEndingDate: moment(createCourseSessionTableInput.sessionEndingDate, "YYYY-MM-DD").format("DD-MM-YYYY"),
        CourseTable: {
          courseId: createCourseSessionTableInput.courseId
        }
      }
    })

    return result;
  }

  async findAll() {
    return await this.courseSessionTableRepository.find({
      relations: {
        CourseTable: true
      }
    })
  }


  async findAllCourseByCourseId(courseId: number) {
    return await this.courseSessionTableRepository.find({
      where: {
        CourseTable: {
          courseId
        },
        isEnrollmentOpen: 1
      }
    })
  }

  async findOne(courseSessionId: number) {
    return await this.courseSessionTableRepository.findOne({
      where: {
        courseSessionId
      },
      relations: {
        CourseTable: true
      }
    })
  }

  async findOneByCourseId(courseId: number) {
    return await this.courseSessionTableRepository.find({
      where: {
        CourseTable: {
          courseId
        }
      },
      relations: {
        CourseTable: true
      }
    })
  }

  async update(courseSessionId: number, updateCourseSessionTableInput: UpdateCourseSessionTableInput) {

    let result = await this.courseSessionTableRepository.findOne({
      where: {
        courseSessionId
      }
    })

    if (result) {
      result.sessionName = updateCourseSessionTableInput.sessionName;
      result.capacity = updateCourseSessionTableInput.capacity;
      result.sessionStartingDate = moment(updateCourseSessionTableInput.sessionStartingDate, "YYYY-MM-DD").format("DD-MM-YYYY");
      result.sessionEndingDate = moment(updateCourseSessionTableInput.sessionEndingDate, "YYYY-MM-DD").format("DD-MM-YYYY");
      result.CourseTable = await this.courseTableService.findOne(updateCourseSessionTableInput.courseId)

      await this.courseSessionTableRepository.save(result)

      return {
        status: true,
        status_text: "Successfully Updated"
      }
    } else {
      return {
        status: false,
        status_text: "Not found"
      }
    }

  }

  async remove(courseSessionId: number) {
    let result = await this.courseSessionTableRepository.findOne({
      where: {
        courseSessionId
      }
    })

    if (result) {
      await this.courseSessionTableRepository.remove(result)

      return {
        status: true,
        status_text: "Successfully Deleted"
      }
    } else {
      return {
        status: false,
        status_text: "Not found"
      }
    }
  }

  async toggleStatusofCourseSession(courseSessionId: number){
    let res = await this.courseSessionTableRepository.findOne({
      where: {
        courseSessionId
      }
    })


    if (res) {
      // update
      let oldStatus = res.isEnrollmentOpen;

      let newStatus = 1;
      if (oldStatus == 0) {
        newStatus = 1;
      }

      if (oldStatus == 1) {
        newStatus = 0;
      }

      res.isEnrollmentOpen = newStatus;

      let result = await this.courseSessionTableRepository.save(res);


      return {
        status: true,
        status_text: "Successfully Updated",
        data: result
      }

    } else {
      // return error
      return {
        status: false,
        status_text: "Not found"
      }
    }
  }
}
