import { Resolver, Query, Mutation, Args, Int, Subscription } from '@nestjs/graphql';
import { CourseTableService } from './course-table.service';
import { CourseTable } from './entities/course-table.entity';
import { CreateCourseTableInput } from './dto/create-course-table.input';
import { UpdateCourseTableInput } from './dto/update-course-table.input';
import { Inject, UseGuards } from '@nestjs/common';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { CourseTableReturnType } from './types/course-table-return.type';
import { CourseTableType } from './types/course-table.type';
import { CountChangedType } from 'src/types/count-changed.type';
import { CurrentUser } from 'src/user-collection/decorators/user.decorator';

export const NEW_COURSE_ADD = "NEW_COURSE_ADD"

@Resolver(() => CourseTable)
export class CourseTableResolver {
  constructor(private readonly courseTableService: CourseTableService,
    @Inject(PUB_SUB) private pubSub: RedisPubSub
    ) {}

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)  
  @Mutation(() => CourseTableReturnType)
  createCourseTable(@Args('createCourseTableInput') createCourseTableInput: CreateCourseTableInput) {
    return this.courseTableService.create(createCourseTableInput);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [CourseTableType])
  findAllCourses() {
    return this.courseTableService.findAll();
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [CourseTableType])
  findAllActiveCourses() {
    return this.courseTableService.findAllActiveCourses();
  }


  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [CourseTableType])
  findMyActiveCourses(@CurrentUser() user: any) {
    return this.courseTableService.findMyActiveCourses(user);
  }


  // @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  // @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [CourseTableType])
  searchActiveCourses( @Args('search', { type: () => String }) search: string) {
    return this.courseTableService.searchActiveCourses(search);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => CourseTableType)
  findOneCourse(@Args('courseId', { type: () => Int }) courseId: number) {
    return this.courseTableService.findOne(courseId);
  }


  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => CourseTableType)
  findOneCourseRestricted(@Args('courseId', { type: () => Int }) courseId: number, @CurrentUser() user : any) {
    return this.courseTableService.findOneRestrictedNew(courseId, user);
  }


  // @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  // @UseGuards(GqlAuthGuard, RolesGuard)
  // @Query(() => CourseTableType)
  // findOneCourseRestrictedNew(@Args('courseId', { type: () => Int }) courseId: number, @CurrentUser() user : any) {
  //   return this.courseTableService.findOneRestrictedNew(courseId, user);
  // }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => CourseTableReturnType)
  updateCourseTable(@Args('updateCourseTableInput') updateCourseTableInput: UpdateCourseTableInput) {
    return this.courseTableService.update(updateCourseTableInput.courseId, updateCourseTableInput);
  }


  @Roles(Role.SUPERADMIN, Role.ADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => CourseTableReturnType)
  toggleStatusofCourse(@Args('courseId', { type: () => Int }) courseId: number) {
    return this.courseTableService.toggleStatusofCourse(courseId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => CourseTableReturnType)
  toggleHasAssesmentofCourse(@Args('courseId', { type: () => Int }) courseId: number) {
    return this.courseTableService.toggleHasAssesmentofCourse(courseId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => CourseTableReturnType)
  removeCourseTable(@Args('courseId', { type: () => Int }) courseId: number) {
    return this.courseTableService.remove(courseId);
  }

  @Subscription(() => CountChangedType, {
    resolve: (payload) => {
      return { countChanged: payload }
    }
  })
  courseSubscription() {
    return this.pubSub.asyncIterator(NEW_COURSE_ADD);
  }
}
