import { Inject, Injectable, forwardRef } from '@nestjs/common';
import { CreateCourseTableInput } from './dto/create-course-table.input';
import { UpdateCourseTableInput } from './dto/update-course-table.input';
import { InjectRepository } from '@nestjs/typeorm';
import { CourseTable } from './entities/course-table.entity';
import { Repository } from 'typeorm';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { FileTableService } from 'src/file-table/file-table.service';
import { SubscriptionTableService } from 'src/subscription-table/subscription-table.service';
import { CourseUnitTableService } from 'src/course-unit-table/course-unit-table.service';
import { CourseProgressionTableService } from 'src/course-progression-table/course-progression-table.service';
import { CourseUnitTable } from 'src/course-unit-table/entities/course-unit-table.entity';
import { UserCollectionService } from 'src/user-collection/user-collection.service';
import { CourseSessionTableService } from 'src/course-session-table/course-session-table.service';

export const NEW_COURSE_ADD = "NEW_COURSE_ADD"

@Injectable()
export class CourseTableService {
  constructor(
    @InjectRepository(CourseTable)
    private readonly courseTableRepository: Repository<CourseTable>,

    @Inject(PUB_SUB) private pubSub: RedisPubSub,

    @Inject(forwardRef(() => FileTableService))
    private readonly fileTableService: FileTableService,

    @Inject(forwardRef(() => SubscriptionTableService))
    private readonly subscriptionTableService: SubscriptionTableService,

    @Inject(forwardRef(() => CourseUnitTableService))
    private readonly courseUnitTableService: CourseUnitTableService,

    @Inject(forwardRef(() => CourseProgressionTableService))
    private readonly courseProgressionTableService: CourseProgressionTableService,


    @Inject(forwardRef(() => UserCollectionService))
    private readonly userCollectionService: UserCollectionService,


    @Inject(forwardRef(() => CourseSessionTableService))
    private readonly courseSessionTableService: CourseSessionTableService,

  ) { }

  async create(createCourseTableInput: CreateCourseTableInput) {
    let checkIfAlreadyExists = await this.checkIfAlreadyExists(createCourseTableInput);

    if (checkIfAlreadyExists == 0) {

      let st = new CourseTable();
      st.courseName = createCourseTableInput.courseName;
      st.courseCode = createCourseTableInput.courseCode;
      st.courseDescription = createCourseTableInput.courseDescription;
      st.courseDuration = createCourseTableInput.courseDuration;
      st.courseType = createCourseTableInput.courseType;
      st.hasAssesment = createCourseTableInput.hasAssesment;
      st.isActive = createCourseTableInput.isActive;
      st.UserTable = await this.userCollectionService.getOneByUserId(createCourseTableInput.userId)
      let result = await this.courseTableRepository.save(st);


      // send to subscriber
      this.pubSub.publish(NEW_COURSE_ADD, 1);


      return {
        status: true,
        status_text: "Successfully Added",
        data: result
      }

    } else {
      return {
        status: false,
        status_text: "Already Exists"
      }
    }
  }

  async checkIfAlreadyExists(createCourseTableInput: CreateCourseTableInput) {

    let result = await this.courseTableRepository.count({
      where: {
        courseCode: createCourseTableInput.courseCode,

      }
    })

    return result;

  }

  async findAll() {
    let result = await this.courseTableRepository.find({
      relations: {
        UserTable: true
      }
    });

    let index = 0;
    for (const r of result) {
      // get the file link

      let url = await this.fileTableService.getUrlOfFileId(r.courseBanner);
      result[index]['courseBannerUrl'] = url;
      index++;
    }

    return result;

  }


  async findAllActiveCourses() {
    let result = await this.courseTableRepository.find({
      where: {
        isActive: "YES"
      },
      relations: {
        CourseUnitTable: true
      }
    });

    let index = 0;
    for (const r of result) {
      // get the file link

      let url = await this.fileTableService.getUrlOfFileId(r.courseBanner);
      result[index]['courseBannerUrl'] = url;
      result[index]['courseUnitCount'] = r.CourseUnitTable.length;

      let totalPages = 0;
      r.CourseUnitTable.map(el => {
        totalPages += el.numberOfPages;
      })

      result[index]['courseUnitPages'] = totalPages;

      index++;
    }

    return result;

  }

  async findMyActiveCourses(user: any) {
    let result = await this.courseTableRepository.find({
      where: {
        isActive: "YES",
        SubscriptionTable: {
          UserTable: {
            id: user.id
          },
          status: "YES"
        }
      },
      relations: {
        CourseUnitTable: true
      }
    });

    let index = 0;
    for (const r of result) {
      // get the file link

      let url = await this.fileTableService.getUrlOfFileId(r.courseBanner);
      result[index]['courseBannerUrl'] = url;
      result[index]['courseUnitCount'] = r.CourseUnitTable.length;

      let totalPages = 0;
      r.CourseUnitTable.map(el => {
        totalPages += el.numberOfPages;
      })

      result[index]['courseUnitPages'] = totalPages;

      index++;
    }

    return result;

  }


  async searchActiveCourses(search: string) {

    let result: any = await this.courseTableRepository.createQueryBuilder('r')
      .andWhere("r.isActive='YES'")
    // .leftJoinAndMapMany(
    //   'r.CourseUnitTable',
    //   CourseUnitTable,
    //   'ml2',
    //   'ml2.courseId=r.courseId',
    // );

    if (search != '') {
      let s = `${search}%`;

      result = await result.andWhere('r.courseName LIKE :courseName', { courseName: s });

    }

    result = await result.getMany();
    console.log("🚀 ~ CourseTableService ~ searchActiveCourses ~ result:", result)



    let index = 0;
    for (const r of result) {
      // get the file link

      let url = await this.fileTableService.getUrlOfFileId(r.courseBanner);
      result[index]['courseBannerUrl'] = url;
      //result[index]['courseUnitCount'] = r.CourseUnitTable.length;

      let totalPages = 0;
      // r.CourseUnitTable.map(el => {
      //   totalPages += el.numberOfPages;
      // })

      result[index]['courseUnitPages'] = totalPages;

      index++;
    }

    return result;

  }

  async findOne(courseId: number) {
    let result: any = await this.courseTableRepository.findOne({
      where: {
        courseId
      },
      relations: {
        CourseUnitTable: true
      }
    })



    if (result) {
      // get the file link

      let url = await this.fileTableService.getUrlOfFileId(result.courseBanner);
      result.courseBannerUrl = url;
      result.courseUnitCount = result.CourseUnitTable.length;
      let totalPages = 0;
      result.CourseUnitTable.map(el => {
        totalPages += el.numberOfPages;
      })

      result.courseUnitPages = totalPages;
    }

    return result;
  }


  async findOneRestricted(courseId: number, user: any) {

    // check whether this course is enrolled to me or not


    let isAllowed = await this.subscriptionTableService.checkIfAllowed(courseId, user.id)




    let result: any = await this.courseTableRepository.findOne({
      where: {
        courseId
      }
    })



    if (result) {
      // get the file link
      let CourseUnitTable = await this.courseUnitTableService.findOneCourseUnitByCourseId(result.courseId)
      result.CourseUnitTable = CourseUnitTable;



      let url = await this.fileTableService.getUrlOfFileId(result.courseBanner);
      result.courseBannerUrl = url;


      result.courseUnitCount = result.CourseUnitTable.length;
      let totalPages = 0;
      result.CourseUnitTable.map(el => {
        totalPages += el.numberOfPages;
      })

      result.courseUnitPages = totalPages;
      result.isAllowed = isAllowed

      // check for session which is now active for registration
      let sessions = await this.courseSessionTableService.findAllCourseByCourseId(result.courseId)
      result.CourseSessionTable = sessions;
      // now get the progression of this course unit

      let index = 0;
      let previousUnitId = 0;
      for (const unit of result.CourseUnitTable) {
        let progression = await this.courseProgressionTableService.getProgressionReport(unit.courseUnitId, user)
        result.CourseUnitTable[index]['CourseProgressionTable'] = progression;

        let isAllowedUnit = false;
        if (previousUnitId == 0) {
          // so this is the first course
          isAllowedUnit = true
        } else {
          // check if previous unit is completed or not

          let previousProgression = await this.courseProgressionTableService.getProgressionReport(previousUnitId, user)
          if (previousProgression && previousProgression.assesmentStatus == 1) {
            isAllowedUnit = true;
          }
        }

        result.CourseUnitTable[index]['isAllowedUnit'] = isAllowedUnit;
        // we have to check is Allowed for all the units

        previousUnitId = unit.courseUnitId;
        index++;
      }


    }

    return result;
  }


  async findOneRestrictedNew(courseId: number, user: any) {

    // check whether this course is enrolled to me or not


    let isAllowed = await this.subscriptionTableService.checkIfAllowed(courseId, user.id)

    let subscriptionDetails = await this.subscriptionTableService.getDetails(courseId, user.id);


    let result: any = await this.courseTableRepository.findOne({
      where: {
        courseId
      },
      relations: {
        AssessmentMasterTable: true
      }
    })



    if (result) {
      // get the file link
      let CourseUnitTable = await this.courseUnitTableService.findOneCourseUnitByCourseId(result.courseId)
      result.CourseUnitTable = CourseUnitTable;



      let url = await this.fileTableService.getUrlOfFileId(result.courseBanner);
      result.courseBannerUrl = url;


      result.courseUnitCount = result.CourseUnitTable.length;
      let totalPages = 0;
      result.CourseUnitTable.map(el => {
        totalPages += el.numberOfPages;
      })

      result.courseUnitPages = totalPages;
      result.isAllowed = isAllowed

      // check for session which is now active for registration
      let sessions = await this.courseSessionTableService.findAllCourseByCourseId(result.courseId)
      result.CourseSessionTable = sessions;
      // now get the progression of this course unit

      let index = 0;
      let previousUnitId = 0;
      for (const unit of result.CourseUnitTable) {
        let progression = await this.courseProgressionTableService.getProgressionReport(unit.courseUnitId, user)
        result.CourseUnitTable[index]['CourseProgressionTable'] = progression;


        // we need to check if the week is okay for this
        // we have staringDate and endingDate from the subscription
        let isAllowedUnit = false;
        if (subscriptionDetails) {
          if (subscriptionDetails.currentWeek >= unit.week) {
            isAllowedUnit = true;
          }
        }


        // if(previousUnitId == 0){
        //   // so this is the first course
        //   isAllowedUnit = true
        // }else{
        //   // check if previous unit is completed or not

        //   let previousProgression = await this.courseProgressionTableService.getProgressionReport(previousUnitId, user)
        //   if(previousProgression && previousProgression.assesmentStatus == 1){
        //     isAllowedUnit = true;
        //   }
        // }

        result.CourseUnitTable[index]['isAllowedUnit'] = isAllowedUnit;
        // we have to check is Allowed for all the units

        previousUnitId = unit.courseUnitId;
        index++;
      }


    }



    return result;
  }


  async updateCourseBanner(courseCode: string, fileId: string) {
    let result = await this.courseTableRepository.findOne({
      where: {
        courseCode
      }
    })

    if (result) {
      result.courseBanner = fileId;

      await this.courseTableRepository.save(result)

      return true;
    } else {
      return false;
    }
  }

  async update(courseId: number, updateCourseTableInput: UpdateCourseTableInput) {
    let result = await this.findOne(courseId);

    if (result) {



      result.courseName = updateCourseTableInput.courseName;
      result.courseCode = updateCourseTableInput.courseCode;
      result.courseDescription = updateCourseTableInput.courseDescription;
      result.courseDuration = updateCourseTableInput.courseDuration;
      result.courseType = updateCourseTableInput.courseType;
      result.hasAssesment = updateCourseTableInput.hasAssesment;
      result.isActive = updateCourseTableInput.isActive;
      result.UserTable = await this.userCollectionService.getOneByUserId(updateCourseTableInput.userId)
      let updatedResult = await this.courseTableRepository.save(result);


      // send to subscriber
      this.pubSub.publish(NEW_COURSE_ADD, 1);


      return {
        status: true,
        status_text: "Successfully Updated",
        data: updatedResult
      }
    } else {
      return {
        status: false,
        status_text: "Not Found"
      }
    }
  }

  async remove(courseId: number) {
    let result = await this.findOne(courseId)

    if (result) {
      await this.courseTableRepository.remove(result);


      // send to subscriber
      this.pubSub.publish(NEW_COURSE_ADD, 1);


      return {
        status: true,
        status_text: "Successfully Deleted"
      }
    } else {
      return {
        status: false,
        status_text: "Not Found"
      }
    }
  }

  async toggleStatusofCourse(courseId: number) {
    let res = await this.courseTableRepository.findOne({
      where: {
        courseId
      }
    })


    if (res) {
      // update
      let oldStatus = res.isActive;

      let newStatus = "YES";
      if (oldStatus == "YES") {
        newStatus = "NO";
      }

      if (oldStatus == "NO") {
        newStatus = "YES";
      }

      res.isActive = newStatus;

      let result = await this.courseTableRepository.save(res);


      return {
        status: true,
        status_text: "Successfully Updated",
        data: result
      }

    } else {
      // return error
      return {
        status: false,
        status_text: "Not found"
      }
    }
  }


  async toggleHasAssesmentofCourse(courseId: number) {
    let res = await this.courseTableRepository.findOne({
      where: {
        courseId
      }
    })


    if (res) {
      // update
      let oldStatus = res.hasAssesment;

      let newStatus = "YES";
      if (oldStatus == "YES") {
        newStatus = "NO";
      }

      if (oldStatus == "NO") {
        newStatus = "YES";
      }

      res.hasAssesment = newStatus;

      let result = await this.courseTableRepository.save(res);


      return {
        status: true,
        status_text: "Successfully Updated",
        data: result
      }

    } else {
      // return error
      return {
        status: false,
        status_text: "Not found"
      }
    }
  }
}
