import { Resolver, Query, Mutation, Args, Int, Subscription } from '@nestjs/graphql';
import { DesignationTableService } from './designation-table.service';
import { DesignationTable } from './entities/designation-table.entity';
import { CreateDesignationTableInput } from './dto/create-designation-table.input';
import { UpdateDesignationTableInput } from './dto/update-designation-table.input';
import { Roles } from 'src/user-collection/decorators/roles.decorator';
import { Role } from 'src/user-collection/roles/role.enum';
import { Inject, UseGuards } from '@nestjs/common';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { DesignationTableReturnType } from './types/designation-table-return.type';
import { DesignationTableType } from './types/designation-table.type';
import { CountChangedType } from 'src/types/count-changed.type';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { PUB_SUB } from 'src/helper/GlobalConstants';


export const NEW_DESIGNATION_ADD = "NEW_DESIGNATION_ADD"

@Resolver(() => DesignationTable)
export class DesignationTableResolver {
  constructor(private readonly designationTableService: DesignationTableService,
    @Inject(PUB_SUB) private pubSub: RedisPubSub) { }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => DesignationTableReturnType)
  createDesignationTable(@Args('createDesignationTableInput') createDesignationTableInput: CreateDesignationTableInput) {
    return this.designationTableService.create(createDesignationTableInput);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [DesignationTableType])
  findAllDesignations() {
    return this.designationTableService.findAll();
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => DesignationTableType)
  findOneDesignation(@Args('designationId', { type: () => Int }) designationId: number) {
    return this.designationTableService.findOne(designationId);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => DesignationTableReturnType)
  updateDesignationTable(@Args('updateDesignationTableInput') updateDesignationTableInput: UpdateDesignationTableInput) {
    return this.designationTableService.update(updateDesignationTableInput.designationId, updateDesignationTableInput);
  }

  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => DesignationTableReturnType)
  removeDesignationTable(@Args('designationId', { type: () => Int }) designationId: number) {
    return this.designationTableService.remove(designationId);
  }


  @Subscription(() => CountChangedType, {
    resolve: (payload) => {
      return { countChanged: payload }
    }
  })
  designationSubscription() {
    return this.pubSub.asyncIterator(NEW_DESIGNATION_ADD);
  }
}
