import { Inject, Injectable, forwardRef } from '@nestjs/common';
import { CreateSubscriptionTableInput } from './dto/create-subscription-table.input';
import { UpdateSubscriptionTableInput } from './dto/update-subscription-table.input';
import { SubscriptionTable } from './entities/subscription-table.entity';
import { Repository } from 'typeorm';
import { InjectRepository } from '@nestjs/typeorm';
import { CourseTableService } from 'src/course-table/course-table.service';
import { UserCollectionService } from 'src/user-collection/user-collection.service';
import moment from 'moment';
import { CourseSessionTableService } from 'src/course-session-table/course-session-table.service';

@Injectable()
export class SubscriptionTableService {
  constructor(
    @InjectRepository(SubscriptionTable)
    private readonly subscriptionTableRepository: Repository<SubscriptionTable>,

    @Inject(forwardRef(() => CourseTableService))
    private readonly courseTableService: CourseTableService,

    @Inject(forwardRef(() => CourseSessionTableService))
    private readonly courseSessionTableService: CourseSessionTableService,

    @Inject(forwardRef(() => UserCollectionService))
    private readonly userCollectionService: UserCollectionService,

  ) { }

  async create(createSubscriptionTableInput: CreateSubscriptionTableInput) {
    let checkIfAlreadyExists = await this.checkIfAlreadyExists(createSubscriptionTableInput);

    if (checkIfAlreadyExists == 0) {

      let st = new SubscriptionTable();

      let courseTable = await this.courseTableService.findOne(createSubscriptionTableInput.courseId)
      let userTable = await this.userCollectionService.findOne(createSubscriptionTableInput.userId)
      let courseSessionTable = await this.courseSessionTableService.findOne(createSubscriptionTableInput.courseSessionId)

      st.CourseTable = courseTable;
      st.UserTable = userTable;
      st.CourseSessionTable = courseSessionTable;

      st.startDate = courseSessionTable.sessionStartingDate;
      st.endDate = courseSessionTable.sessionEndingDate;
      st.entryDate = moment().format('DD-MM-YYYY');
      st.status = createSubscriptionTableInput.status;


      let result = await this.subscriptionTableRepository.save(st);


      return {
        status: true,
        status_text: "Successfully Added",
        data: result
      }

    } else {
      return {
        status: false,
        status_text: "Already Exists"
      }
    }
  }

  async checkIfAlreadyExists(createSubscriptionTableInput: CreateSubscriptionTableInput) {

    let result = await this.subscriptionTableRepository.count({
      where: {
        CourseTable: {
          courseId: createSubscriptionTableInput.courseId
        },
        CourseSessionTable: {
          courseSessionId: createSubscriptionTableInput.courseSessionId
        },
        UserTable: {
          id: createSubscriptionTableInput.userId
        },
        // startDate: createSubscriptionTableInput.startDate,
        // endDate: createSubscriptionTableInput.endDate,
      }
    })

    return result;

  }

  async findAll() {
    let result = await this.subscriptionTableRepository.find({
      relations: {
        CourseTable: true,
        UserTable: true,
        CourseSessionTable: true
      }
    });
    return result;

  }



  async findAllSubscriptionsByCourseId(courseId: number) {
    let result = await this.subscriptionTableRepository.find({
      where: {
        CourseTable: {
          courseId
        }
      },
      relations: {
        CourseTable: true,
        UserTable: true
      }
    });
    return result;

  }



  async findOne(subscriptionId: number) {
    let result: any = await this.subscriptionTableRepository.findOne({
      where: {
        subscriptionId
      },
      relations: {
        CourseTable: true,
        UserTable: true
      }
    })

    return result;
  }


  async enrollMyself(courseId: number, courseSessionId: number, user: any) {
    let result = await this.subscriptionTableRepository.findOne({
      where: {
        CourseTable: {
          courseId
        },
        CourseSessionTable: {
          courseSessionId
        },
        UserTable: {
          id: user.id
        },
        status: "YES",
        isComplete: 0
      },
    });

    if (result) {
      return {
        status: false,
        status_text: "Already subscribed"
      }
    } else {
      let subT = new SubscriptionTable();
      let courseTable = await this.courseTableService.findOne(courseId)
      let userTable = await this.userCollectionService.findOne(user.id)
      let courseSessionTable = await this.courseSessionTableService.findOne(courseSessionId)

      subT.CourseTable = courseTable;
      subT.UserTable = userTable;
      subT.CourseSessionTable = courseSessionTable;

      subT.startDate = courseSessionTable.sessionStartingDate;
      subT.endDate = courseSessionTable.sessionEndingDate;
      subT.entryDate = moment().format('DD-MM-YYYY');
      subT.status = "YES";
      subT.isComplete = 0;

      let updatedResult = await this.subscriptionTableRepository.save(subT);
      return {
        status: true,
        status_text: "Successfully Subscribed",
        data: updatedResult
      }
    }





  }


  async markAsComplete(courseId: number, courseSessionId: number, user: any) {
    let result = await this.subscriptionTableRepository.findOne({
      where: {
        CourseTable: {
          courseId
        },
        CourseSessionTable: {
          courseSessionId
        },
        UserTable: {
          id: user.id
        },
        isComplete: 0
      },
    });

    if (result) {
      result.isComplete = 1;
      await this.subscriptionTableRepository.save(result);
      return {
        status: true,
        status_text: "Course is complete"
      }
    } else {
      return {
        status: false,
        status_text: "No Subscription found"
      }
    }





  }


  async checkIfAllowed(courseId: number, userId: number) {
    let result = await this.subscriptionTableRepository.findOne({
      where: {
        CourseTable: {
          courseId
        },
        UserTable: {
          id: userId
        },
      }
    })






    if (result) {

      // check the dates

      if (moment(result.endDate, "DD-MM-YYYY").isBefore(moment())) {
        return false;
      }
      if (result.isComplete == 1) {
        return true;
      }

      if (result.isComplete == 0 && result.status == "YES") {
        return true
      }

      return false;
    } else {
      return false;
    }
  }


  divideDateRangeIntoWeeks(startDate: string, endDate: string) {
    // Parse the start and end dates using Moment.js
    let start = moment(startDate, 'DD-MM-YYYY');
    let end = moment(endDate, 'DD-MM-YYYY');

    // Calculate the total number of days in the date range
    let totalDays = end.diff(start, 'days');

    // Calculate the length of each week
    let baseWeekLength = Math.floor(totalDays / 4);
    let extraDays = totalDays % 4;

    // Create arrays to hold the start and end dates of each week
    let weeks = [];
    let currentStart = start.clone();

    for (let i = 0; i < 4; i++) {
      let weekLength = baseWeekLength;
      if (i === 3) { // Add extra days to the last week
        weekLength += extraDays;
      }

      let weekEnd = currentStart.clone().add(weekLength, 'days');

      weeks.push({
        weekStart: currentStart.format('DD-MM-YYYY'),
        weekEnd: weekEnd.format('DD-MM-YYYY')
      });

      // Move the start to the next day after the current week ends
      currentStart = weekEnd.clone().add(1, 'days');
    }

    return weeks;
  }


  getCurrentWeekNumber(startDate: string, endDate: string) {
    let weeks = this.divideDateRangeIntoWeeks(startDate, endDate);
    let today = moment();

    for (let i = 0; i < weeks.length; i++) {
      if (moment(today).isBetween(moment(weeks[i].weekStart, "DD-MM-YYYY"), moment(weeks[i].weekEnd, "DD-MM-YYYY"), null, '[]')) {
        return i + 1; // Week numbers are 1-indexed
      }
    }
    return -1; // Return -1 if today is not within the range
  }


  async getDetails(courseId: number, userId: number) {
    let result: any = await this.subscriptionTableRepository.findOne({
      where: {
        CourseTable: {
          courseId
        },
        UserTable: {
          id: userId
        },
      }
    })

    if (result) {

      console.log(this.divideDateRangeIntoWeeks(result.startDate, result.endDate))

      // we need to devide the starting and ending date in 4 weeks
      result.currentWeek = this.getCurrentWeekNumber(result.startDate, result.endDate);

      // find today is which week



      return result;
    } else {
      return false;
    }
  }


  async update(subscriptionId: number, updateSubscriptionTableInput: UpdateSubscriptionTableInput) {
    let result = await this.findOne(subscriptionId);

    if (result) {



      let courseTable = await this.courseTableService.findOne(updateSubscriptionTableInput.courseId)
      let userTable = await this.userCollectionService.findOne(updateSubscriptionTableInput.userId)
      let courseSessionTable = await this.courseSessionTableService.findOne(updateSubscriptionTableInput.courseSessionId)

      result.CourseTable = courseTable;
      result.UserTable = userTable;
      result.CourseSessionTable = courseSessionTable;

      result.startDate = courseSessionTable.sessionStartingDate;
      result.endDate = courseSessionTable.sessionEndingDate;
      result.entryDate = moment().format('DD-MM-YYYY');
      result.status = updateSubscriptionTableInput.status;

      let updatedResult = await this.subscriptionTableRepository.save(result);




      return {
        status: true,
        status_text: "Successfully Updated",
        data: updatedResult
      }
    } else {
      return {
        status: false,
        status_text: "Not Found"
      }
    }
  }

  async remove(subscriptionId: number) {
    let result = await this.findOne(subscriptionId)

    if (result) {
      await this.subscriptionTableRepository.remove(result);



      return {
        status: true,
        status_text: "Successfully Deleted"
      }
    } else {
      return {
        status: false,
        status_text: "Not Found"
      }
    }
  }


  async toggleStatusofSubscription(subscriptionId: number) {
    let res = await this.subscriptionTableRepository.findOne({
      where: {
        subscriptionId
      }
    })


    if (res) {
      // update
      let oldStatus = res.status;

      let newStatus = "YES";
      if (oldStatus == "YES") {
        newStatus = "NO";
      }

      if (oldStatus == "NO") {
        newStatus = "YES";
      }

      res.status = newStatus;

      let result = await this.subscriptionTableRepository.save(res);


      return {
        status: true,
        status_text: "Successfully Updated",
        data: result
      }

    } else {
      // return error
      return {
        status: false,
        status_text: "Not found"
      }
    }
  }
}
