import { Resolver, Query, Mutation, Args, Int, Subscription } from '@nestjs/graphql';
import { UserCollectionService } from './user-collection.service';
import { UserTable } from './entities/user-collection.entity';
import { CreateUserCollectionInput } from './dto/create-user-collection.input';
import { UpdateUserCollectionInput } from './dto/update-user-collection.input';
import { UserCollectionType } from './type/user-collection.type';
// import { LoginDto } from './dto/login-dto';
import { ForgotPasswordType } from './type/forgot-password.type';
import { Roles } from './decorators/roles.decorator';
import { Role } from './roles/role.enum';
import { Inject, UseGuards } from '@nestjs/common';
import { GqlAuthGuard } from 'src/auth/guards/gql-auth.guard';
import { RolesGuard } from 'src/auth/guards/roles.guard';
import { UserCollectionReturnType } from './type/user-collection-return.type';
import { NewUserAddInput } from './dto/add-nerw-user.input';
import { CurrentUser } from './decorators/user.decorator';
import { Connection, EntityManager } from 'typeorm';
import { UserPaginationType } from './type/UserPaginationType';
import { OtpTableService } from 'src/otp-table/otp-table.service';
import { DashboardCountType } from './type/dashboard-count.type';
import { PUB_SUB } from 'src/helper/GlobalConstants';
import { RedisPubSub } from 'graphql-redis-subscriptions';
import { CountChangedType } from 'src/types/count-changed.type';
import { UserIntegratorCheckType } from './type/user-integrator-check.type';
import { NewRegistrationInput } from './dto/new-registration.input';

export const NEW_USER_ADD = "NEW_USER_ADD"
@Resolver(() => UserTable)
export class UserCollectionResolver {
  constructor(private readonly userCollectionService: UserCollectionService, private connection: Connection, private entityManager: EntityManager,
    private readonly otpTableService: OtpTableService,
    @Inject(PUB_SUB) private pubSub: RedisPubSub
  ) { }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => UserCollectionReturnType)
  createUserCollection(@Args('createUserCollectionInput') createUserCollectionInput: CreateUserCollectionInput) {
    return this.userCollectionService.create(createUserCollectionInput);
  }


  // @Roles(Role.SUPERADMIN)
  // @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => UserCollectionReturnType)
  newRegistrationFromApp(@Args('newRegistrationInput') newRegistrationInput: NewRegistrationInput) {
    return this.userCollectionService.newRegistrationFromApp(newRegistrationInput);
  }





  @Roles(Role.SUPERADMIN, Role.ADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [UserCollectionType])
  findAllUsers() {
    return this.userCollectionService.findAll();
  }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => UserCollectionType)
  findOne(@Args('id', { type: () => Int }) id: number) {
    return this.userCollectionService.findOne(id);
  }


  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => UserIntegratorCheckType)
  checkIfUserIsIntegrator(@CurrentUser() user: any) {
    return this.userCollectionService.checkIfUserIsIntegrator(user);
  }

  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => UserCollectionReturnType)
  updateUserCollection(@Args('updateUserCollectionInput') updateUserCollectionInput: UpdateUserCollectionInput) {
    return this.userCollectionService.update(updateUserCollectionInput.id, updateUserCollectionInput);
  }

  // @Roles(Role.ADMIN)
  // @UseGuards(GqlAuthGuard, RolesGuard)
  // @Mutation(() => UserCollectionReturnType)
  // removeUserCollection(@Args('id', { type: () => Int }) id: number) {
  //   return this.userCollectionService.remove(id);
  // }


  @Mutation(() => ForgotPasswordType)
  async forgotPassword(@Args('username') username: string) {
    // get user-phone and generate an otp and sent to the user via SMS
    let result = await this.userCollectionService.getUserByUsername(username);
    if (result !== null && result !== undefined) {
      // get the phone number and generate OTP
      let resultStr = await this.otpTableService.generateOtpNow(result.phone, result.name, result.email);
      return {
        status: true,
        status_text: resultStr.status_text,

      }
    } else {
      return {
        status: false,
        status_text: 'There is no entry for this username',

      }
    }
  }


  @Mutation(() => ForgotPasswordType)
  async forgotPasswordCheckOtp(@Args('username') username: string, @Args('otp') otp: string) {
    // get user-phone and generate an otp and sent to the user via SMS
    let result = await this.userCollectionService.getUserByUsername(username);
    if (result !== null && result !== undefined) {
      // get the phone number and generate OTP
      return await this.otpTableService.checkOtp(result.phone, otp);

    } else {
      return {
        status: false,
        status_text: 'There is no entry for this email address',
        data: false
      }
    }


  }

  @Mutation(() => ForgotPasswordType)
  async forgotPasswordFinalSubmit(@Args('username') username: string, @Args('otp') otp: string, @Args('password') password: string) {
    // get user-phone and generate an otp and sent to the user via SMS
    let result = await this.userCollectionService.getUserByUsername(username);
    if (result !== null && result !== undefined) {
      // get the phone number and generate OTP
      let res = await this.otpTableService.checkOtp(result.phone, otp);
      if (res.status === true) {
        // update the password
        return await this.userCollectionService.changePassword(username, password);


      } else {
        return {
          status: false,
          status_text: 'Operation Failed',
          data: false
        }
      }

    } else {
      return {
        status: false,
        status_text: 'There is no entry for this email address',
        data: false
      }
    }


  }


  @UseGuards(GqlAuthGuard)
  @Mutation(() => ForgotPasswordType)
  changePasswordSecuredByUser(@Args('oldPassword') oldPassword: string, @Args('newPassword') newPassword: string, @CurrentUser() user: any) {
    return this.userCollectionService.changePasswordSecured(user, oldPassword, newPassword);
  }


  // @Mutation(() => ForgotPasswordType)
  // changePassword(@Args('password') password: string, @Args('data') data: string) {
  //   return this.userCollectionService.changePassword(data, password);
  // }

  // @Roles(Role.SUPERADMIN)
  // @UseGuards(GqlAuthGuard, RolesGuard)
  // @Query(() => [UserCollectionType])
  // findOneClientUsers(
  //   @Args('id')
  //   id: number,
  // ) {
  //   return this.userCollectionService.findOneUsersByClientId(id);
  // }


  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => UserCollectionReturnType)
  toggleActiveStatus(
    @Args('id')
    id: number,
  ) {
    return this.userCollectionService.toggleActiveStatus(id);
  }


  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => UserCollectionReturnType)
  changePassword(
    @CurrentUser() user: any,
    @Args('password')
    password: string,
  ) {
    return this.userCollectionService.changePassword(user.username, password);
  }






  // @Roles(Role.ADMIN, Role.COLLECTOR)
  // @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => UserCollectionReturnType)
  async addNewUser(
    @Args('newUserAddInput')
    newUserAddInput: NewUserAddInput
  ) {

    return await this.userCollectionService.addNewUser(newUserAddInput);




  }


  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => UserCollectionType)
  async getUserByUsername(
    @Args('username')
    username: string
  ) {
    return await this.userCollectionService.getUserByUsername(username);
  }


  @Roles(Role.SUPERADMIN, Role.ADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => UserCollectionReturnType)
  async addIntegratorToUser(
    @Args('integratorId') integratorId: number, @Args('id') id: number
  ) {
    return await this.userCollectionService.addIntegratorToUser(id, integratorId);
  }



  @Roles(Role.SUPERADMIN, Role.ADMIN, Role.SUPERVISOR, Role.USER)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => UserCollectionType)
  async getMyUserDetails(
    @CurrentUser()
    user: any
  ) {
    return await this.userCollectionService.getUserByUsername(user.username);

  }


  // @Roles(Role.ADMIN, Role.COLLECTOR)
  // @UseGuards(GqlAuthGuard, RolesGuard)
  // @Query(() => UserPaginationType)
  // getAllUsers(@Args('limit') limit: number, @Args('page') page: number, @Args('search') search: string) {
  //   return this.userCollectionService.getAllUsers(limit, page, search);

  // }

  @Roles(Role.SUPERADMIN, Role.ADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Query(() => [UserCollectionType])
  getAllUsers() {
    return this.userCollectionService.getAllUsersNew();

  }


  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => UserCollectionReturnType)
  removeUserCollection(
    @Args('id')
    id: number
  ) {
    return this.userCollectionService.deleteUser(id);
  }




  @Roles(Role.SUPERADMIN)
  @UseGuards(GqlAuthGuard, RolesGuard)
  @Mutation(() => ForgotPasswordType)
  changePasswordSecured(@Args('oldPassword') oldPassword: string, @Args('newPassword') newPassword: string, @CurrentUser() user: any) {
    return this.userCollectionService.changePasswordSecured(user, oldPassword, newPassword);
  }



  @Subscription(() => CountChangedType, {
    resolve: (payload) => {
      return { countChanged: payload }
    }
  })
  userSubscription() {
    return this.pubSub.asyncIterator(NEW_USER_ADD);
  }






}